/* global config, Notifier, session */

/**
 * Ajax call
 * 
 * @param {String}      path
 * @param {Object}      params
 * @param {Function}    callback
 * @param {String}      type
 * @param {Array}       headers
 */
function aj_call(path, params, callback, type='POST', headers = {}) {
    $.ajax({
        url: path,
        type: type,
        data: params,
        headers: headers,
        success:callback,
        error: function(data) {
            activeBtn && activeBtn.prop('disabled', false);
            Notifier.error(data.responseJSON.error.description);
        }
    });
}

/**
 * Makes a rest call to the plugin backend
 * 
 * @param {String}      route
 * @param {Object}      params
 * @param {Function}    callback
 * @param {String}      type
 */
function rest_call(route, params, callback, type='POST') {
    aj_call((config.rest_url + route), params, callback, type, {
        "X-WP-Nonce": session.nonce
    });
}

/**
 * Loads administration page
 * 
 * @param {String} name
 */
function loadPage(name) {
    console.log('will load ' + name);
    $('#cnd-content').empty().load(config.pages_url + name + '.html');
}

/**
 * Serializes a form
 * 
 * @param   {String} id
 * @returns {Array}
 */
function serializeForm(id){
    let form = $('#cnd-content #'+id);
    var unindexed_array = form.serializeArray();
    var indexed_array = {};

    $.map(unindexed_array, function(n, i){
        indexed_array[n['name']] = n['value'];
    });

    return indexed_array;
}

/**
 * Converts unix timestamp to textual date
 * 
 * @param   {Int}       unix        the unix timestamp
 * @param   {Boolean}   relative    [optional] whether to use relative formatting (today, yesterday..). default = true.
 * @param   {Boolean}   time        [optional] whether to display time as well. default = false.
 * @param   {Boolean}   year        [optional] whether to display year . default = true.
 * @returns {String}
 */
function getEnglishDate(unix, relative = true, time = false, year = true) {
    var date = unix ? new Date(unix*1000) : new Date();
    var refdate = new Date();
    var value='';
    if(relative) {
        var d = new Date(date);
        if(d.setHours(0,0,0,0) === refdate.setHours(0,0,0,0))
            value = 'Today';
        refdate.setDate(refdate.getDate() - 1);
        if(d.setHours(0,0,0,0) === refdate.setHours(0,0,0,0))
            value = 'Yesterday';
        refdate.setDate(refdate.getDate() - 1);
        if(d.setHours(0,0,0,0) === refdate.setHours(0,0,0,0))
            value = '2 days ago';
        refdate.setDate(refdate.getDate() + 3);
        if(d.setHours(0,0,0,0) === refdate.setHours(0,0,0,0))
            value = 'Tomorrow';
    }
    if(value.length === 0) {
        var refdate = new Date();
        var months = ['Januery','February','March','April','May','June','July','August','September','October','November','December'];
        value = months[date.getMonth()]+' '+date.getDate()+getDateSuffix()+' ' + (year ? (date.getFullYear() === refdate.getFullYear() ? '' : ' '+date.getFullYear()) : '');
    }
    if(time) {
        value += ' at '+(date.getHours())+':'+("0" + date.getMinutes()).slice(-2);
    }
    
    return value;
    
    function getDateSuffix() {
        switch(date.getDate()) {
            case '1' :
                return 'st';
            case '2' :
                return 'nd';
            case '3' :
                return 'rd';
            default:
                return 'th';
        }
    }
}

/**
 * Format a number
 * 
 * @param   {Number}    n
 * @param   {Int}       fixed   Number of digits after decimal point
 * @returns {String}
 */
function formatNumber(n, fixed) {
    if(fixed === undefined) fixed = 2;
    return parseFloat(n).toFixed(fixed).toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
}

/**
 * Copies elements content to clipboard
 * 
 * @param   {Object} el
 */
function registerCopyToClipboard(el) {
    el.find('.ctcb').unbind('click').click(function() {
        var targetStr = $(this).attr('ctcb-target');
        var target = el.find('#{0}.cpable'.format(targetStr));
        if(target.is('p'))
            var text = target.text();
        else
            var text = target.val();

        success('Copied to clipboard');
        if (window.clipboardData && window.clipboardData.setData)
            return window.clipboardData.setData("Text", text);

        else if (document.queryCommandSupported && document.queryCommandSupported("copy")) {
            var textarea = document.createElement("textarea");
            textarea.textContent = text;
            textarea.style.position = "fixed";
            document.body.appendChild(textarea);
            textarea.select();
            try {
                return document.execCommand("copy");
            }
            catch (ex) {
                console.warn("Copy to clipboard failed.", ex);
                return false;
            }
            finally {
                document.body.removeChild(textarea);
            }
        }
    });
}

/**
 * Prints a success message
 * 
 * @param {String} msg
 */
function success(msg) {
    Notifier.success(msg);
}
