<?php

namespace Candifly;

use Candifly\Exception\AuthenticateException;
use Firebase\JWT\JWT;

if(!defined('ABSPATH'))
    exit();


class CandiMenu
{

    /**
     * Main menu page content
     */
    public function render()
    {
        $root = dirname(plugin_dir_url( __FILE__ ));
        wp_enqueue_style('cnd_mainmenu_css', $root.'/css/mainmenu.css');
        wp_enqueue_style('google_fonts', 'https://fonts.googleapis.com/css?family=Nunito:200,200i,300,300i,400,400i,600,600i,700,700i,800,800i,900,900i');
        wp_enqueue_style('bootstrap_css', 'https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css');
        wp_enqueue_style('fontawesome', 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/4.7.0/css/font-awesome.min.css');
        wp_enqueue_script('cnd_admin_funcs', $root.'/js/funcs.js', [], false, true );
        wp_enqueue_script('cnd_admin_notifier', $root.'/js/lib/notifier/Notifier.js', [], false, true );
        wp_enqueue_script('cnd_admin_bootbox', $root.'/js/lib/bootbox/bootbox.min.js', [], false, true );
        wp_enqueue_script('popper', 'https://cdn.jsdelivr.net/npm/@popperjs/core@2.9.2/dist/umd/popper.min.js');
        wp_enqueue_script('bootstrap_js', 'https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/js/bootstrap.min.js', [], false, true );
        
        $token = $this->getValidToken();
        
        ?>
            <div class="cnd-header">
                <img src="<?php echo $root; ?>/img/icon.svg" class="cnd-header-logo">
                <h1>CANDIFLY</h1>
            </div>
            <div class="row">
                <div class="col-lg-9">
                    <div id="cnd-content">
                    </div>
                </div>
                <div class="col-lg-3">
                    <div class='cnd-sidebar'>
                        <img src="<?php echo $root; ?>/img/icon.svg" class="cnd-icon-logo"><small><a href="https://candifly.com" target="_blank">Candifly</a>&nbsp; WordPress plugin version <?php echo Application::getConfig()->ver; ?></small>
                        
                        <h5>Support</h5>
                        <p>Questions? suggestions? Don't be a stranger!<br>We love to hear feedback</p>
                        <ul>
                            <li>Send us an email at <a href="mailto:contact@candifly.com" target="_blank">contact@candifly.com</a></li>
                            <li>Open a new topic on our <a href="https://wordpress.org/support/plugin/candifly/" target="_blank">Plugin forum</a></li>
                            <li>Use the <a href="<?php echo Application::getConfig()->candifly_app; ?>/contact" target="_blank">Contact us</a> page on Candifly dashboard</li>
                        </ul>
                        <p>Visit our <a href="https://candifly.com/faq" target="_blank">FAQ</a> page to get immediate help on various topics.</p>
                        
                        <h5>Enjoy this plugin?</h5>
                        <p>Our basic product is 100% free and includes features that you get only with premium accounts elsewhere.
                            <br><br>You can show your appreciation by -
                        </p>
                        <ul>
                            <li><a href="https://wordpress.org/support/plugin/candifly/reviews/#new-post" target="_blank">Reviewing our plugin</a></li>
                            <li>Tell your friends about us</li>
                            <li>Upgrade to a <a href="https://candifly.com/plans" target="_blank">premium account</a></li>
                        </ul>
                        <div id="pluginUpdates"></div>
                    </div>
                </div>
            </div>

            <script>
                var $ = jQuery;
                var config = {
                    admin_url: "<?php echo admin_url(); ?>",
                    plugin_url: "<?php echo $root; ?>",
                    pages_url: "<?php echo $root; ?>/pages/",
                    rest_url: "<?php echo get_rest_url() ?>",
                    candifly_url: "<?php echo Application::getConfig()->candifly_app ?>",
                    wp_email: "<?php $current_user = wp_get_current_user(); echo $current_user->data->user_email; ?>"
                },
                session = {
                    email: "<?php echo get_option(CANDIFLY_EMAIL_OPTION) ?? ''; ?>",
                    secret: "<?php echo get_option(CANDIFLY_SECRET_OPTION) ?? ''; ?>",
                    authKey: "<?php echo $token ?? ''; ?>",
                    nonce: "<?php echo wp_create_nonce('wp_rest'); ?>"
                },
                activeBtn;
                
                $(document).ready(function() {
                    loadPage('<?php echo $this->getLoadPage(); ?>');
                    
                    if (!String.prototype.format) {
                        String.prototype.format = function() {
                            var args = arguments;
                            return this.replace(/{(\d+)}/g, function(match, number) {
                                return typeof args[number] != 'undefined' ? args[number] : match;
                            });
                        };
                    }
                    
                    checkVersionUpdates();
                });
                
                /**
                 * Checks for plugin new version
                 */
                function checkVersionUpdates() {
                    rest_call('candifly/v1/app/updateAvailable', {}, (res) => {
                        if(res.updateAvailable)
                            $('.cnd-sidebar #pluginUpdates').empty().html(
                                '<p style="margin-top:30px;">New plugin version available! Candifly ver. {0}.<br>'.format(res.updateAvailable)+
                                'Click <a href="https://wordpress.org/plugins/candifly" target="_blank">here</a> to update.'
                                );
                    });
                }
                
            </script>
        <?php

        
    }
    
    /**
     * Returns start page
     * 
     * @return string
     */
    private function getLoadPage()
    {
        $secret = get_option(CANDIFLY_SECRET_OPTION);
        return empty($secret) ? 'initial' : 'dashboard';
    }
    
    /**
     * Returns a valid token, if one exists
     * 
     * @return string
     */
    private function getValidToken()
    {
        $token = get_option(CANDIFLY_KEY_OPTION);
        if($token && !$this->validateToken($token)) {
            $secret = get_option(CANDIFLY_SECRET_OPTION);
            if(!$secret)
                return;

            $token = CandiRest::getToken($secret);
            update_option(CANDIFLY_KEY_OPTION, $token);
        }
            
        return $token;
    }
    
    /**
     * Validates JWT token is valid and not expired
     * 
     * @param   string $token
     * @return  boolean
     * @throws  AuthenticateException
     */
    private function validateToken($token)
    {
        try {
            $body64 = \explode('.', $token)[1];
            $payloadRaw = JWT::urlsafeB64Decode($body64);
            if (null === ($payload = JWT::jsonDecode($payloadRaw)))
                throw new AuthenticateException(910004, 'Invalid claims encoding');

            if (\is_array($payload))
                // prevent PHP Fatal Error in edge-cases when payload is empty array
                $payload = (object) $payload;

//            var_dump($payload); var_dump($payload instanceof object); exit();
//            if (!$payload instanceof stdClass)
//                throw new AuthenticateException(910005, 'Payload must be a JSON object');

            if (time() >= $payload->exp)
                return false;

            return true;
            
        } 
        catch (AuthenticateException $ex) {
            return false;
        }
    }

}

