<?php

namespace Candifly\Mapper;


use GuzzleHttp\Client as GuzzleClient;
use GuzzleHttp\Exception\ClientException;
use Psr\Http\Message\ResponseInterface;

use Candifly\Application;
use Candifly\Exception\AuthenticateException;

class ServiceMapper
{
    
    private $config;
    
    /**
     * Sets the service id
     * 
     * @param   string $id
     * @return  ServiceMapper
     * @throws  BadRequestException
     */
    public function setService($id)
    {
        if(!array_key_exists($id, Application::getConfig()->services))
            throw new \Exception(910001, 'can`t find service - '.$id);
        
        $this->config = Application::getConfig()->services->$id;
        
        return $this;
    }
    
    /**
     * Returns a response from the service
     * 
     * @param   string  $method
     * @param   array   $body
     * @return  array
     */
    public function getResponse($method, $body = [])
    {
        if(!$this->config)
            throw new \Exception(910002, 'service id not set');
            
        $methodConfig = $this->config->methods->$method;
        $headers = $methodConfig->headers ? 
                ['Authorization' => 'Bearer '.get_option(CANDIFLY_KEY_OPTION)] :
                [];
        
        $client = new GuzzleClient([
            'headers' => $headers
        ]);
        
        $httpMethod = $methodConfig->HTTPMethod;
        try {
            $response = $client->$httpMethod($this->config->domain.$methodConfig->route, [
                    \GuzzleHttp\RequestOptions::JSON => $body
                ]);
        } 
        catch (ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents();
            $res = json_decode($msg);
            if(property_exists($res, 'success') && $res->success = 'fasle' && property_exists($res, 'error') && property_exists($res->error, 'description'))
                $msg = $res->error->description;
            
            throw new AuthenticateException(920001, $msg);
        }
        
//        var_dump($this->config->domain.$methodConfig->route);
//        var_dump($response); exit();
        return json_decode($response->getBody());
    }
    
    
}